@@ -125,19 +125,19 @@ if LooseVersion(np.__version__) < '1.12.0':
 
         Examples
         --------
-        >>> np.nancumsum(1) #doctest: +SKIP
+        >>> nancumsum(1)
         array([1])
-        >>> np.nancumsum([1]) #doctest: +SKIP
+        >>> nancumsum([1])
         array([1])
-        >>> np.nancumsum([1, np.nan]) #doctest: +SKIP
+        >>> nancumsum([1, np.nan])
         array([ 1.,  1.])
         >>> a = np.array([[1, 2], [3, np.nan]])
-        >>> np.nancumsum(a) #doctest: +SKIP
+        >>> nancumsum(a)
         array([ 1.,  3.,  6.,  6.])
-        >>> np.nancumsum(a, axis=0) #doctest: +SKIP
+        >>> nancumsum(a, axis=0)
         array([[ 1.,  2.],
                [ 4.,  2.]])
-        >>> np.nancumsum(a, axis=1) #doctest: +SKIP
+        >>> nancumsum(a, axis=1)
         array([[ 1.,  3.],
                [ 3.,  3.]])
 
@@ -186,19 +186,19 @@ if LooseVersion(np.__version__) < '1.12.0':
 
         Examples
         --------
-        >>> np.nancumprod(1) #doctest: +SKIP
+        >>> nancumprod(1)
         array([1])
-        >>> np.nancumprod([1]) #doctest: +SKIP
+        >>> nancumprod([1])
         array([1])
-        >>> np.nancumprod([1, np.nan]) #doctest: +SKIP
+        >>> nancumprod([1, np.nan])
         array([ 1.,  1.])
         >>> a = np.array([[1, 2], [3, np.nan]])
-        >>> np.nancumprod(a) #doctest: +SKIP
+        >>> nancumprod(a)
         array([ 1.,  2.,  6.,  6.])
-        >>> np.nancumprod(a, axis=0) #doctest: +SKIP
+        >>> nancumprod(a, axis=0)
         array([[ 1.,  2.],
                [ 3.,  2.]])
-        >>> np.nancumprod(a, axis=1) #doctest: +SKIP
+        >>> nancumprod(a, axis=1)
         array([[ 1.,  2.],
                [ 3.,  3.]])
 
@@ -207,15 +207,141 @@ if LooseVersion(np.__version__) < '1.12.0':
         return np.cumprod(a, axis=axis, dtype=dtype, out=out)
 
 
-def _make_sliced_dtype(dtype, index):
-    if LooseVersion(np.__version__) == LooseVersion("1.14.0"):
-        return _make_sliced_dtype_np_ge_14(dtype, index)
-    else:
-        return _make_sliced_dtype_np_lt_14(dtype, index)
+if LooseVersion(np.__version__) < '1.15.0':
+    # These functions were added in numpy 1.15.0. For previous versions they
+    # are duplicated here
+
+    def _make_along_axis_idx(arr_shape, indices, axis):
+        # compute dimensions to iterate over
+        if not np.issubdtype(indices.dtype, np.integer):
+            raise IndexError('`indices` must be an integer array')
+        if len(arr_shape) != indices.ndim:
+            raise ValueError(
+                "`indices` and `arr` must have the same number of dimensions")
+        shape_ones = (1,) * indices.ndim
+        dest_dims = list(range(axis)) + [None] + list(
+            range(axis + 1, indices.ndim))
+
+        # build a fancy index, consisting of orthogonal aranges, with the
+        # requested index inserted at the right location
+        fancy_index = []
+        for dim, n in zip(dest_dims, arr_shape):
+            if dim is None:
+                fancy_index.append(indices)
+            else:
+                ind_shape = shape_ones[:dim] + (-1,) + shape_ones[dim + 1:]
+                fancy_index.append(np.arange(n).reshape(ind_shape))
+
+        return tuple(fancy_index)
+
+    def take_along_axis(arr, indices, axis):
+        """
+        Take values from the input array by matching 1d index and data slices.
+        This iterates over matching 1d slices oriented along the specified axis in
+        the index and data arrays, and uses the former to look up values in the
+        latter. These slices can be different lengths.
+        Functions returning an index along an axis, like `argsort` and
+        `argpartition`, produce suitable indices for this function.
+        .. versionadded:: 1.15.0
+        Parameters
+        ----------
+        arr: ndarray (Ni..., M, Nk...)
+            Source array
+        indices: ndarray (Ni..., J, Nk...)
+            Indices to take along each 1d slice of `arr`. This must match the
+            dimension of arr, but dimensions Ni and Nj only need to broadcast
+            against `arr`.
+        axis: int
+            The axis to take 1d slices along. If axis is None, the input array is
+            treated as if it had first been flattened to 1d, for consistency with
+            `sort` and `argsort`.
+        Returns
+        -------
+        out: ndarray (Ni..., J, Nk...)
+            The indexed result.
+        Notes
+        -----
+        This is equivalent to (but faster than) the following use of `ndindex` and
+        `s_`, which sets each of ``ii`` and ``kk`` to a tuple of indices::
+            Ni, M, Nk = a.shape[:axis], a.shape[axis], a.shape[axis+1:]
+            J = indices.shape[axis]  # Need not equal M
+            out = np.empty(Nk + (J,) + Nk)
+            for ii in ndindex(Ni):
+                for kk in ndindex(Nk):
+                    a_1d       = a      [ii + s_[:,] + kk]
+                    indices_1d = indices[ii + s_[:,] + kk]
+                    out_1d     = out    [ii + s_[:,] + kk]
+                    for j in range(J):
+                        out_1d[j] = a_1d[indices_1d[j]]
+        Equivalently, eliminating the inner loop, the last two lines would be::
+                    out_1d[:] = a_1d[indices_1d]
+        See Also
+        --------
+        take : Take along an axis, using the same indices for every 1d slice
+        put_along_axis :
+            Put values into the destination array by matching 1d index and data slices
+        Examples
+        --------
+        For this sample array
+        >>> a = np.array([[10, 30, 20], [60, 40, 50]])
+
+        We can sort either by using sort directly, or argsort and this function
+        >>> np.sort(a, axis=1)
+        array([[10, 20, 30],
+               [40, 50, 60]])
+        >>> ai = np.argsort(a, axis=1); ai
+        array([[0, 2, 1],
+               [1, 2, 0]])
+        >>> take_along_axis(a, ai, axis=1)
+        array([[10, 20, 30],
+               [40, 50, 60]])
+
+        The same works for max and min, if you expand the dimensions:
+        >>> np.expand_dims(np.max(a, axis=1), axis=1)
+        array([[30],
+               [60]])
+        >>> ai = np.expand_dims(np.argmax(a, axis=1), axis=1)
+        >>> ai
+        array([[1],
+               [0]])
+        >>> take_along_axis(a, ai, axis=1)
+        array([[30],
+               [60]])
+
+        If we want to get the max and min at the same time,
+        we can stack the indices first:
+        >>> ai_min = np.expand_dims(np.argmin(a, axis=1), axis=1)
+        >>> ai_max = np.expand_dims(np.argmax(a, axis=1), axis=1)
+        >>> ai = np.concatenate([ai_min, ai_max], axis=1)
+        >>> ai
+        array([[0, 1],
+               [1, 0]])
+        >>> take_along_axis(a, ai, axis=1)
+        array([[10, 30],
+               [40, 60]])
+        """
+        # normalize inputs
+        if axis is None:
+            arr = arr.flat
+            arr_shape = (len(arr),)  # flatiter has no .shape
+            axis = 0
+        else:
+            if axis < 0:
+                axis = arr.ndim + axis
+            arr_shape = arr.shape
+
+        # use the fancy index
+        return arr[_make_along_axis_idx(arr_shape, indices, axis)]
 
 
-def _make_sliced_dtype_np_ge_14(dtype, index):
-    # For https://github.com/numpy/numpy/pull/6053, NumPy >= 1.14
+def _make_sliced_dtype_np_ge_16(dtype, index):
+    # This was briefly added in 1.14.0
+    # https://github.com/numpy/numpy/pull/6053, NumPy >= 1.14
+    # which was then reverted in 1.14.1 with
+    # https://github.com/numpy/numpy/pull/10411
+    # And then was finally released with
+    # https://github.com/numpy/numpy/pull/12447
+    # in version 1.16.0
     new = {
         'names': index,
         'formats': [dtype.fields[name][0] for name in index],
@@ -231,6 +357,13 @@ def _make_sliced_dtype_np_lt_14(dtype, index):
     return dt
 
 
+if LooseVersion(np.__version__) >= LooseVersion("1.16.0") or \
+   LooseVersion(np.__version__) == LooseVersion("1.14.0"):
+    _make_sliced_dtype = _make_sliced_dtype_np_ge_16
+else:
+    _make_sliced_dtype = _make_sliced_dtype_np_lt_14
+
+
 class _Recurser(object):
     """
     Utility class for recursing over nested iterables
