@@ -9,12 +9,12 @@ import pickle
 import os
 import threading
 import uuid
-import warnings
 
 from toolz import merge, groupby, curry, identity
 from toolz.functoolz import Compose
 
-from .compatibility import long, unicode, Iterator
+from .compatibility import (apply, long, unicode, Iterator, is_dataclass,
+                            dataclass_fields, Mapping, cPickle)
 from .context import thread_state
 from .core import flatten, quote, get as simple_get
 from .hashing import hash_buffer_hex
@@ -157,11 +157,11 @@ class DaskMethodsMixin(object):
         return result
 
 
-def compute_as_if_collection(cls, dsk, keys, get=None, scheduler=None, **kwargs):
+def compute_as_if_collection(cls, dsk, keys, scheduler=None, get=None, **kwargs):
     """Compute a graph as if it were of type cls.
 
     Allows for applying the same optimizations and default scheduler."""
-    schedule = get_scheduler(get=get, scheduler=scheduler, cls=cls)
+    schedule = get_scheduler(scheduler=scheduler, cls=cls, get=get)
     dsk2 = optimization_function(cls)(ensure_dict(dsk), keys, **kwargs)
     return schedule(dsk2, keys, **kwargs)
 
@@ -190,8 +190,8 @@ def collections_to_dsk(collections, optimize_graph=True, **kwargs):
             groups = {k: (opt(dsk, keys), keys)
                       for k, (dsk, keys) in groups.items()}
 
-        dsk = merge(*(opt(dsk, keys, **kwargs)
-                      for opt, (dsk, keys) in groups.items()))
+        dsk = merge(*map(ensure_dict, [opt(dsk, keys, **kwargs)
+                         for opt, (dsk, keys) in groups.items()]))
     else:
         dsk, _ = _extract_graph_and_keys(collections)
 
@@ -200,19 +200,18 @@ def collections_to_dsk(collections, optimize_graph=True, **kwargs):
 
 def _extract_graph_and_keys(vals):
     """Given a list of dask vals, return a single graph and a list of keys such
-    that ``get(dsk, keys)`` is equivalent to ``[v.compute() v in vals]``."""
-    dsk = {}
-    keys = []
-    for v in vals:
-        d = v.__dask_graph__()
-        if hasattr(d, 'dicts'):
-            for dd in d.dicts.values():
-                dsk.update(dd)
-        else:
-            dsk.update(d)
-        keys.append(v.__dask_keys__())
+    that ``get(dsk, keys)`` is equivalent to ``[v.compute() for v in vals]``."""
+    from .highlevelgraph import HighLevelGraph
 
-    return dsk, keys
+    graphs = [v.__dask_graph__() for v in vals]
+    keys = [v.__dask_keys__() for v in vals]
+
+    if any(isinstance(graph, HighLevelGraph) for graph in graphs):
+        graph = HighLevelGraph.merge(*graphs)
+    else:
+        graph = merge(*graphs)
+
+    return graph, keys
 
 
 def unpack_collections(*args, **kwargs):
@@ -267,6 +266,10 @@ def unpack_collections(*args, **kwargs):
             elif typ is dict:
                 tsk = (dict, [[_unpack(k), _unpack(v)]
                               for k, v in expr.items()])
+            elif is_dataclass(expr):
+                tsk = (apply, typ, (), (dict,
+                       [[f.name, _unpack(getattr(expr, f.name))] for f in
+                        dataclass_fields(expr)]))
             else:
                 return expr
 
@@ -385,9 +388,9 @@ def compute(*args, **kwargs):
     if not collections:
         return args
 
-    schedule = get_scheduler(get=kwargs.pop('get', None),
-                             scheduler=kwargs.pop('scheduler', None),
-                             collections=collections)
+    schedule = get_scheduler(scheduler=kwargs.pop('scheduler', None),
+                             collections=collections,
+                             get=kwargs.pop('get', None))
 
     dsk = collections_to_dsk(collections, optimize_graph, **kwargs)
     keys = [x.__dask_keys__() for x in collections]
@@ -447,10 +450,10 @@ def visualize(*args, **kwargs):
     filename = kwargs.pop('filename', 'mydask')
     optimize_graph = kwargs.pop('optimize_graph', False)
 
-    dsks = [arg for arg in args if isinstance(arg, dict)]
+    dsks = [arg for arg in args if isinstance(arg, Mapping)]
     args = [arg for arg in args if is_dask_collection(arg)]
 
-    dsk = collections_to_dsk(args, optimize_graph=optimize_graph)
+    dsk = dict(collections_to_dsk(args, optimize_graph=optimize_graph))
     for d in dsks:
         dsk.update(d)
 
@@ -546,8 +549,7 @@ def persist(*args, **kwargs):
     if not collections:
         return args
 
-    schedule = get_scheduler(get=kwargs.pop('get', None),
-                             scheduler=kwargs.pop('scheduler', None),
+    schedule = get_scheduler(scheduler=kwargs.pop('scheduler', None),
                              collections=collections)
 
     if inspect.ismethod(schedule):
@@ -727,10 +729,18 @@ def register_numpy():
                     x.shape, x.strides, offset)
         if x.dtype.hasobject:
             try:
-                data = hash_buffer_hex('-'.join(x.flat).encode('utf-8'))
+                # string fast-path
+                data = hash_buffer_hex('-'.join(x.flat).encode(encoding='utf-8', errors='surrogatepass'))
+            except UnicodeDecodeError:
+                # bytes fast-path
+                data = hash_buffer_hex(b'-'.join(x.flat))
             except TypeError:
-                data = hash_buffer_hex(b'-'.join([unicode(item).encode('utf-8') for item in
-                                                  x.flat]))
+                # object data w/o fast-path, use fast cPickle
+                try:
+                    data = hash_buffer_hex(cPickle.dumps(x, cPickle.HIGHEST_PROTOCOL))
+                except Exception:
+                    # pickling not supported, use UUID4-based fallback
+                    data = uuid.uuid4().hex
         else:
             try:
                 data = hash_buffer_hex(x.ravel(order='K').view('i1'))
@@ -816,22 +826,24 @@ else:
     })
 
 
-_warnned_on_get = [False]
+get_err_msg = """
+The get= keyword has been removed.
 
+Please use the scheduler= keyword instead with the name of
+the desired scheduler like 'threads' or 'processes'
 
-def warn_on_get(get):
-    if _warnned_on_get[0]:
-        return
-    else:
-        if get in named_schedulers.values():
-            _warnned_on_get[0] = True
-            warnings.warn(
-                "The get= keyword has been deprecated. "
-                "Please use the scheduler= keyword instead with the name of "
-                "the desired scheduler like 'threads' or 'processes'\n"
-                "    x.compute(scheduler='threads') \n"
-                "or with a function that takes the graph and keys\n"
-                "    x.compute(scheduler=my_scheduler_function)")
+    x.compute(scheduler='single-threaded')
+    x.compute(scheduler='threads')
+    x.compute(scheduler='processes')
+
+or with a function that takes the graph and keys
+
+    x.compute(scheduler=my_scheduler_function)
+
+or with a Dask client
+
+    x.compute(scheduler=client)
+""".strip()
 
 
 def get_scheduler(get=None, scheduler=None, collections=None, cls=None):
@@ -839,19 +851,15 @@ def get_scheduler(get=None, scheduler=None, collections=None, cls=None):
 
     There are various ways to specify the scheduler to use:
 
-    1.  Passing in get= parameters (deprecated)
-    2.  Passing in scheduler= parameters
-    3.  Passing these into global confiuration
-    4.  Using defaults of a dask collection
+    1.  Passing in scheduler= parameters
+    2.  Passing these into global confiuration
+    3.  Using defaults of a dask collection
 
     This function centralizes the logic to determine the right scheduler to use
     from those many options
     """
-    if get is not None:
-        if scheduler is not None:
-            raise ValueError("Both get= and scheduler= provided.  Choose one")
-        warn_on_get(get)
-        return get
+    if get:
+        raise TypeError(get_err_msg)
 
     if scheduler is not None:
         if callable(scheduler):
@@ -863,6 +871,8 @@ def get_scheduler(get=None, scheduler=None, collections=None, cls=None):
         elif scheduler.lower() in ('dask.distributed', 'distributed'):
             from distributed.worker import get_client
             return get_client().get
+        elif scheduler.lower() in ['processes', 'multiprocessing']:
+            raise ValueError("Please install cloudpickle to use the '%s' scheduler." % scheduler)
         else:
             raise ValueError("Expected one of [distributed, %s]" % ', '.join(sorted(named_schedulers)))
         # else:  # try to connect to remote scheduler with this name
@@ -872,8 +882,7 @@ def get_scheduler(get=None, scheduler=None, collections=None, cls=None):
         return get_scheduler(scheduler=config.get('scheduler', None))
 
     if config.get('get', None):
-        warn_on_get(config.get('get', None))
-        return config.get('get', None)
+        raise ValueError(get_err_msg)
 
     if getattr(thread_state, 'key', False):
         from distributed.worker import get_worker
@@ -890,7 +899,7 @@ def get_scheduler(get=None, scheduler=None, collections=None, cls=None):
             raise ValueError("Compute called on multiple collections with "
                              "differing default schedulers. Please specify a "
                              "scheduler=` parameter explicitly in compute or "
-                             "globally with `set_options`.")
+                             "globally with `dask.config.set`.")
         return get
 
     return None
