@@ -3,6 +3,7 @@ from __future__ import absolute_import, division, print_function
 from collections import defaultdict
 import pandas as pd
 from toolz import partition_all
+from numbers import Integral
 
 from ..base import tokenize, compute_as_if_collection
 from .accessor import Accessor
@@ -105,7 +106,7 @@ def categorize(df, columns=None, index=None, split_every=None, **kwargs):
         split_every = 16
     elif split_every is False:
         split_every = df.npartitions
-    elif not isinstance(split_every, int) or split_every < 2:
+    elif not isinstance(split_every, Integral) or split_every < 2:
         raise ValueError("split_every must be an integer >= 2")
 
     token = tokenize(df, columns, index, split_every)
@@ -244,6 +245,10 @@ class CategoricalAccessor(Accessor):
 
         # Reorder to keep cat:code relationship, filtering unused (-1)
         ordered, mask = present.reindex(meta_cat.categories)
+        if mask is None:
+            # PANDAS-23963: old and new categories match.
+            return self._series
+
         new_categories = ordered[mask != -1]
         meta = meta_cat.set_categories(new_categories, ordered=meta_cat.ordered)
         return self._series.map_partitions(self._delegate_method, 'cat',
