@@ -583,7 +583,7 @@ def _write_fastparquet(df, fs, fs_token, path, write_index=None, append=False,
             raise ValueError('Appended columns not the same.\n'
                              'Previous: {} | New: {}'
                              .format(pf.columns, list(df.columns)))
-        elif set(pf.dtypes[c] for c in pf.columns) != set(df[pf.columns].dtypes):
+        elif (pd.Series(pf.dtypes).loc[pf.columns] != df[pf.columns].dtypes).any():
             raise ValueError('Appended dtypes differ.\n{}'
                              .format(set(pf.dtypes.items()) ^
                                      set(df.dtypes.iteritems())))
@@ -659,9 +659,6 @@ def _read_pyarrow(fs, fs_token, paths, columns=None, filters=None,
     # We would like to resolve these to the correct dataframe names
     # as soon as possible.
 
-    if filters is not None:
-        raise NotImplementedError("Predicate pushdown not implemented")
-
     if isinstance(categories, string_types):
         categories = [categories]
     elif categories is None:
@@ -672,7 +669,8 @@ def _read_pyarrow(fs, fs_token, paths, columns=None, filters=None,
     if isinstance(columns, tuple):
         columns = list(columns)
 
-    dataset = pq.ParquetDataset(paths, filesystem=get_pyarrow_filesystem(fs))
+    dataset = pq.ParquetDataset(paths, filesystem=get_pyarrow_filesystem(fs),
+                                filters=filters)
     if dataset.partitions is not None:
         partitions = [n for n in dataset.partitions.partition_names
                       if n is not None]
@@ -907,8 +905,10 @@ def _read_pyarrow_parquet_piece(fs, piece, columns, index_cols, is_series,
         df = table.to_pandas()
         for cat in categories:
             df[cat] = df[cat].astype('category')
-    else:
+    elif pa.__version__ < LooseVersion('0.11.0'):
         df = table.to_pandas(categories=categories)
+    else:
+        df = table.to_pandas(categories=categories, date_as_object=False)
     has_index = not isinstance(df.index, pd.RangeIndex)
 
     if not has_index and index_cols:
@@ -1122,7 +1122,7 @@ def read_parquet(path, columns=None, filters=None, categories=None, index=None,
                     'when reading from fastparquet.ParquetFile')
             is_ParquetFile = True
     except ImportError:
-            pass
+        pass
 
     if is_ParquetFile:
         read = get_engine('fastparquet')['read']
@@ -1137,6 +1137,7 @@ def read_parquet(path, columns=None, filters=None, categories=None, index=None,
             mode='rb',
             storage_options=storage_options
         )
+        paths = path
     else:
         read = get_engine(engine)['read']
         fs, fs_token, paths = get_fs_token_paths(
