@@ -56,14 +56,14 @@ We proceed with hash joins in the following stages:
 from __future__ import absolute_import, division, print_function
 
 from functools import wraps, partial
-from warnings import warn
+import warnings
 
 from toolz import merge_sorted, unique, first
-import toolz
 import pandas as pd
 
 from ..base import tokenize
 from ..compatibility import apply
+from ..highlevelgraph import HighLevelGraph
 from .core import (_Frame, DataFrame, Series, map_partitions, Index,
                    _maybe_from_pandas, new_dd_object, is_broadcastable)
 from .io import from_pandas
@@ -233,8 +233,8 @@ def merge_indexed_dataframes(lhs, rhs, how='left', lsuffix='', rsuffix='',
                     left_index=left_index, right_index=right_index,
                     left_on=left_on, right_on=right_on,
                     suffixes=(lsuffix, rsuffix), indicator=indicator)
-    return new_dd_object(toolz.merge(lhs.dask, rhs.dask, dsk),
-                         name, meta, divisions)
+    graph = HighLevelGraph.from_collections(name, dsk, dependencies=[lhs, rhs])
+    return new_dd_object(graph, name, meta, divisions)
 
 
 shuffle_func = shuffle  # name sometimes conflicts with keyword argument
@@ -291,8 +291,8 @@ def hash_join(lhs, left_on, rhs, right_on, how='inner',
            for i in range(npartitions)}
 
     divisions = [None] * (npartitions + 1)
-    return new_dd_object(toolz.merge(lhs2.dask, rhs2.dask, dsk),
-                         name, meta, divisions)
+    graph = HighLevelGraph.from_collections(name, dsk, dependencies=[lhs2, rhs2])
+    return new_dd_object(graph, name, meta, divisions)
 
 
 def single_partition_join(left, right, **kwargs):
@@ -323,8 +323,8 @@ def single_partition_join(left, right, **kwargs):
         else:
             divisions = [None for _ in left.divisions]
 
-    return new_dd_object(toolz.merge(dsk, left.dask, right.dask), name,
-                         meta, divisions)
+    graph = HighLevelGraph.from_collections(name, dsk, dependencies=[left, right])
+    return new_dd_object(graph, name, meta, divisions)
 
 
 @wraps(pd.merge)
@@ -439,13 +439,15 @@ def concat_unindexed_dataframes(dfs):
 
     meta = pd.concat([df._meta for df in dfs], axis=1)
 
-    return new_dd_object(toolz.merge(dsk, *[df.dask for df in dfs]),
-                         name, meta, dfs[0].divisions)
+    graph = HighLevelGraph.from_collections(name, dsk, dependencies=dfs)
+    return new_dd_object(graph, name, meta, dfs[0].divisions)
 
 
 def concat_indexed_dataframes(dfs, axis=0, join='outer'):
     """ Concatenate indexed dataframes together along the index """
-    meta = methods.concat([df._meta for df in dfs], axis=axis, join=join)
+    warn = axis != 0
+    meta = methods.concat([df._meta for df in dfs], axis=axis, join=join,
+                          filter_warning=warn)
     empties = [strip_unknown_categories(df._meta) for df in dfs]
 
     dfs2, divisions, parts = align_partitions(*dfs)
@@ -456,7 +458,11 @@ def concat_indexed_dataframes(dfs, axis=0, join='outer'):
                for df, empty in zip(part, empties)]
               for part in parts]
 
-    dsk = dict(((name, i), (methods.concat, part, axis, join))
+    filter_warning = True
+    uniform = False
+
+    dsk = dict(((name, i), (methods.concat, part, axis, join,
+                            uniform, filter_warning))
                for i, part in enumerate(parts2))
     for df in dfs2:
         dsk.update(df.dask)
@@ -466,7 +472,7 @@ def concat_indexed_dataframes(dfs, axis=0, join='outer'):
 
 def stack_partitions(dfs, divisions, join='outer'):
     """Concatenate partitions on axis=0 by doing a simple stack"""
-    meta = methods.concat([df._meta for df in dfs], join=join)
+    meta = methods.concat([df._meta for df in dfs], join=join, filter_warning=False)
     empty = strip_unknown_categories(meta)
 
     name = 'concat-{0}'.format(tokenize(*dfs))
@@ -599,9 +605,10 @@ def concat(dfs, axis=0, join='outer', interleave_partitions=False):
         elif (len(dasks) == len(dfs) and
               all(not df.known_divisions for df in dfs) and
               len({df.npartitions for df in dasks}) == 1):
-            warn("Concatenating dataframes with unknown divisions.\n"
-                 "We're assuming that the indexes of each dataframes are \n"
-                 "aligned. This assumption is not generally safe.")
+            warnings.warn("Concatenating dataframes with unknown divisions.\n"
+                          "We're assuming that the indexes of each dataframes"
+                          " are \n aligned. This assumption is not generally "
+                          "safe.")
             return concat_unindexed_dataframes(dfs)
         else:
             raise ValueError('Unable to concatenate DataFrame with unknown '
