@@ -1,3 +1,4 @@
+import itertools
 import os
 import sys
 import pandas as pd
@@ -152,7 +153,7 @@ def test_partitioning_index_categorical_on_values():
     assert (res == res2).all()
 
 
-@pytest.mark.parametrize('npartitions', [1, 4, 7, pytest.mark.slow(23)])
+@pytest.mark.parametrize('npartitions', [1, 4, 7, pytest.param(23, marks=pytest.mark.slow)])
 def test_set_index_tasks(npartitions):
     df = pd.DataFrame({'x': np.random.random(100),
                        'y': np.random.random(100) // 0.2},
@@ -258,8 +259,9 @@ def test_rearrange(shuffle, scheduler):
     a = result.compute(scheduler=scheduler)
     get = dask.base.get_scheduler(scheduler=scheduler)
     parts = get(result.dask, result.__dask_keys__())
+
     for i in a.y.drop_duplicates():
-        assert sum(i in part.y for part in parts) == 1
+        assert sum(i in set(part.y) for part in parts) == 1
 
 
 def test_rearrange_by_column_with_narrow_divisions():
@@ -755,3 +757,28 @@ def test_gh_2730():
         result.sort_values('KEY').reset_index(drop=True),
         expected
     )
+
+
+@pytest.mark.parametrize('npartitions', [None, 'auto'])
+def test_set_index_does_not_repeat_work_due_to_optimizations(npartitions):
+    # Atomic counter
+    count = itertools.count()
+
+    def increment():
+        next(count)
+
+    def make_part(dummy, n):
+        return pd.DataFrame({'x': np.random.random(n),
+                             'y': np.random.random(n)})
+
+    nbytes = 1e6
+    nparts = 50
+    n = int(nbytes / (nparts * 8))
+
+    dsk = {('inc', i): (increment,) for i in range(nparts)}
+    dsk.update({('x', i): (make_part, ('inc', i), n) for i in range(nparts)})
+    ddf = dd.DataFrame(dsk, 'x', make_part(None, 1), [None] * (nparts + 1))
+
+    ddf.set_index('x', npartitions=npartitions)
+    ntimes = next(count)
+    assert ntimes == nparts
