@@ -497,18 +497,18 @@ def _nonempty_series(s, idx=None):
 
 def is_dataframe_like(df):
     """ Looks like a Pandas DataFrame """
-    return set(dir(df)) > {'dtypes', 'columns', 'groupby', 'head'}
+    return set(dir(df)) > {'dtypes', 'columns', 'groupby', 'head'} and not isinstance(df, type)
 
 
 def is_series_like(s):
     """ Looks like a Pandas Series """
-    return set(dir(s)) > {'name', 'dtype', 'groupby', 'head'}
+    return set(dir(s)) > {'name', 'dtype', 'groupby', 'head'} and not isinstance(s, type)
 
 
 def is_index_like(s):
     """ Looks like a Pandas Index """
     attrs = set(dir(s))
-    return attrs > {'name', 'dtype'} and 'head' not in attrs
+    return attrs > {'name', 'dtype'} and 'head' not in attrs and not isinstance(s, type)
 
 
 def check_meta(x, meta, funcname=None, numeric_equal=True):
@@ -536,7 +536,7 @@ def check_meta(x, meta, funcname=None, numeric_equal=True):
     def equal_dtypes(a, b):
         if is_categorical_dtype(a) != is_categorical_dtype(b):
             return False
-        if (a is '-' or b is '-'):
+        if isinstance(a, str) and a == '-' or isinstance(b, str) and b == '-':
             return False
         if is_categorical_dtype(a) and is_categorical_dtype(b):
             # Pandas 0.21 CategoricalDtype compat
