@@ -13,7 +13,6 @@ from importlib import import_module
 from numbers import Integral
 from threading import Lock
 import uuid
-import warnings
 from weakref import WeakValueDictionary
 
 from .compatibility import (get_named_args, getargspec, PY3, unicode,
@@ -439,7 +438,10 @@ def _skip_doctest(line):
     if stripped == '>>>' or stripped.startswith('>>> #'):
         return stripped
     elif '>>>' in stripped and '+SKIP' not in stripped:
-        return line + '  # doctest: +SKIP'
+        if '# doctest:' in line:
+            return line + ', +SKIP'
+        else:
+            return line + '  # doctest: +SKIP'
     else:
         return line
 
@@ -778,11 +780,11 @@ class SerializableLock(object):
             self.lock = Lock()
             SerializableLock._locks[self.token] = self.lock
 
-    def acquire(self, *args):
-        return self.lock.acquire(*args)
+    def acquire(self, *args, **kwargs):
+        return self.lock.acquire(*args, **kwargs)
 
-    def release(self, *args):
-        return self.lock.release(*args)
+    def release(self, *args, **kwargs):
+        return self.lock.release(*args, **kwargs)
 
     def __enter__(self):
         self.lock.__enter__()
@@ -790,9 +792,8 @@ class SerializableLock(object):
     def __exit__(self, *args):
         self.lock.__exit__(*args)
 
-    @property
     def locked(self):
-        return self.locked
+        return self.lock.locked()
 
     def __getstate__(self):
         return self.token
@@ -806,13 +807,12 @@ class SerializableLock(object):
     __repr__ = __str__
 
 
-def get_scheduler_lock(get=None, collection=None, scheduler=None):
+def get_scheduler_lock(collection=None, scheduler=None):
     """Get an instance of the appropriate lock for a certain situation based on
        scheduler used."""
     from . import multiprocessing
     from .base import get_scheduler
-    actual_get = get_scheduler(get=get,
-                               collections=[collection],
+    actual_get = get_scheduler(collections=[collection],
                                scheduler=scheduler)
 
     if actual_get == multiprocessing.get:
@@ -935,7 +935,7 @@ def natural_sort_key(s):
     ['f0', 'f1', 'f2', 'f8', 'f9', 'f10', 'f11', 'f19', 'f20', 'f21']
     """
     return [int(part) if part.isdigit() else part
-            for part in re.split('(\d+)', s)]
+            for part in re.split(r'(\d+)', s)]
 
 
 def factors(n):
@@ -1016,14 +1016,6 @@ byte_sizes.update({k[0]: v for k, v in byte_sizes.items() if k and 'i' not in k}
 byte_sizes.update({k[:-1]: v for k, v in byte_sizes.items() if k and 'i' in k})
 
 
-def effective_get(get=None, collection=None):
-    """ Deprecated: see dask.base.get_scheduler """
-    warnings.warn("Deprecated, see dask.base.get_scheduler instead")
-
-    from dask.base import get_scheduler
-    return get_scheduler(get=get, collections=[collection])
-
-
 def has_keyword(func, keyword):
     try:
         if PY3:
@@ -1035,3 +1027,12 @@ def has_keyword(func, keyword):
                 return keyword in inspect.getargspec(func).args
     except Exception:
         return False
+
+
+def ndimlist(seq):
+    if not isinstance(seq, (list, tuple)):
+        return 0
+    elif not seq:
+        return 1
+    else:
+        return 1 + ndimlist(seq[0])
